@@ -16,6 +16,7 @@
 #include <map>
 #include <string>
 #include <utility>
+#include <variant>
 #include <vector>
 
 #include <caf/actor_system.hpp>
@@ -24,19 +25,16 @@
 #include <caf/downstream.hpp>
 #include <caf/error.hpp>
 #include <caf/event_based_actor.hpp>
-#include <caf/io/accept_handle.hpp>
-#include <caf/io/connection_handle.hpp>
-#include <caf/io/middleman.hpp>
-#include <caf/io/network/test_multiplexer.hpp>
 #include <caf/logger.hpp>
 #include <caf/scheduler/test_coordinator.hpp>
-#include <caf/test/io_dsl.hpp>
 #include <caf/timestamp.hpp>
-#include <caf/variant.hpp>
 
 #include "broker/configuration.hh"
 #include "broker/endpoint.hh"
 #include "broker/error.hh"
+#include "broker/internal/core_actor.hh"
+#include "broker/internal/endpoint_access.hh"
+#include "broker/internal/type_id.hh"
 #include "broker/peer_info.hh"
 #include "broker/peer_status.hh"
 #include "broker/status.hh"
@@ -44,13 +42,12 @@
 #include "broker/subscriber.hh"
 #include "broker/topic.hh"
 
+namespace atom = broker::internal::atom;
+
 using namespace broker;
 
 using caf::unit_t;
 
-using caf::io::accept_handle;
-using caf::io::connection_handle;
-
 namespace {
 
 struct peer_fixture;
@@ -95,59 +92,39 @@ struct peer_fixture {
   // Convenient access to `sys.scheduler()` with proper type.
   caf::scheduler::test_coordinator& sched;
 
-  // Convenienct access to `sys.middleman()`.
-  caf::io::middleman& mm;
-
-  // Convenient access to `mm.backend()` with proper type.
-  caf::io::network::test_multiplexer& mpx;
-
-  // Lists all open connections on this peer.
-  std::vector<connection_handle> connections;
-
-  // Lists all open "ports" on this peer.
-  std::vector<accept_handle> acceptors;
-
   // Stores all received items for subscribed topics.
   std::vector<data_message> data;
 
   // Stores the interval between two credit rounds.
   caf::timespan credit_round_interval;
 
+  // Returns the core manager for given core actor.
+  auto& state(caf::actor hdl) {
+    auto ptr = caf::actor_cast<caf::abstract_actor*>(hdl);
+    return dynamic_cast<internal::core_actor&>(*ptr).state;
+  }
+
   // Initializes this peer and registers it at parent.
   peer_fixture(global_fixture* parent_ptr, std::string peer_name)
     : parent(parent_ptr),
       name(std::move(peer_name)),
       ep(base_fixture::make_config()),
-      sys(ep.system()),
-      sched(dynamic_cast<caf::scheduler::test_coordinator&>(sys.scheduler())),
-      mm(sys.middleman()),
-      mpx(dynamic_cast<caf::io::network::test_multiplexer&>(mm.backend())) {
+      sys(broker::internal::endpoint_access{&ep}.sys()),
+      sched(dynamic_cast<caf::scheduler::test_coordinator&>(sys.scheduler())) {
     // Register at parent.
     parent->peers.emplace(name, this);
     // Run initialization code
     exec_loop();
+    // Register at parent.
+    parent->peers.emplace(name, this);
   }
 
   ~peer_fixture() {
-    CAF_SET_LOGGER_SYS(&ep.system());
+    CAF_SET_LOGGER_SYS(&sys);
     MESSAGE("shut down " << name);
     loop_after_all_enqueues();
   }
 
-  // Returns the next unused connection handle.
-  connection_handle make_connection_handle() {
-    auto result = connection_handle::from_int(parent->next_handle_id++);
-    connections.emplace_back(result);
-    return result;
-  }
-
-  // Returns the next unused accept handle.
-  accept_handle make_accept_handle() {
-    auto result = accept_handle::from_int(parent->next_handle_id++);
-    acceptors.emplace_back(result);
-    return result;
-  }
-
   std::vector<peer_info> peers() {
     sched.inline_next_enqueue();
     return ep.peers();
@@ -155,7 +132,7 @@ struct peer_fixture {
 
   // Subscribes to a topic, storing all incoming tuples in `data`.
   void subscribe_to(topic t) {
-    ep.subscribe_nosync(
+    ep.subscribe(
       {t},
       [](unit_t&) {
         // nop
@@ -163,7 +140,7 @@ struct peer_fixture {
       [=](unit_t&, data_message x) {
         this->data.emplace_back(std::move(x));
       },
-      [](unit_t&, const caf::error&) {
+      [](unit_t&, const error&) {
         // nop
       }
     );
@@ -173,24 +150,7 @@ struct peer_fixture {
   // Publishes all `(t, xs)...` tuples.
   template <class... Ts>
   void publish(topic t, Ts... xs) {
-    using buf_t = std::deque<data_message>;
-    auto buf
-      = std::make_shared<buf_t>(buf_t{make_data_message(t, std::move(xs))...});
-    ep.publish_all_nosync(
-      [](unit_t&) {
-        // nop
-      },
-      [=](unit_t&, caf::downstream<data_message>& out, size_t num) {
-        auto n = std::min(num, buf->size());
-        CAF_MESSAGE("push" << n << "values downstream");
-        for (size_t i = 0u; i < n; ++i)
-          out.push(buf->at(i));
-        buf->erase(buf->begin(), buf->begin() + static_cast<ptrdiff_t>(n));
-      },
-      [=](const unit_t&) {
-        return buf->empty();
-      }
-    );
+    (ep.publish(make_data_message(t, std::move(xs))), ...);
     parent->exec_loop();
   }
 
@@ -228,15 +188,25 @@ bool global_fixture::try_exec() {
 }
 
 void global_fixture::exec_loop() {
-  /*
-  while (try_exec())
+  auto try_trigger_timeout = [this] {
+    std::vector<caf::actor_clock::duration_type> ts;
+    for (auto& kvp : peers) {
+      auto& tac = kvp.second->sched.clock();
+      if (!tac.schedule().empty())
+        ts.emplace_back(tac.schedule().begin()->first - tac.now());
+    }
+    if (!ts.empty()) {
+      auto dt = std::min_element(ts.begin(), ts.end());
+      for (auto& kvp : peers)
+        kvp.second->sched.clock().advance_time(*dt);
+      return true;
+    } else {
+      return false;
+    }
+  };
+  auto exec = [](auto& kvp) { return kvp.second->try_exec(); };
+  while (std::any_of(peers.begin(), peers.end(), exec) || try_trigger_timeout())
     ; // rinse and repeat
-  // */
-  std::vector<peer_fixture*> xs;
-  for (auto& kvp : peers)
-    xs.emplace_back(kvp.second);
-  exec_all_fixtures(xs.begin(), xs.end());
-  // */
 }
 
 // A fixture for simple setups consisting of three nodes.
@@ -249,13 +219,9 @@ struct triangle_fixture : global_fixture {
     : mercury(this, "mercury"),
       venus(this, "venus"),
       earth(this, "earth") {
-    base_fixture::init_socket_api();
-  }
-
-  ~triangle_fixture() {
-    base_fixture::deinit_socket_api();
   }
 
+  // Connect mercury to venus and earth.
   void connect_peers() {
     MESSAGE("prepare connections");
     auto server_handle = mercury.make_accept_handle();
@@ -300,12 +266,12 @@ CAF_TEST(topic_prefix_matching_async_subscribe) {
   CAF_REQUIRE_EQUAL(mercury_peers.size(), 2u);
   CAF_CHECK_EQUAL(mercury_peers.front().status, peer_status::peered);
   CAF_CHECK_EQUAL(mercury_peers.back().status, peer_status::peered);
-  MESSAGE("assume one peer for venus");
+  MESSAGE("assume two peers for venus");
   venus.loop_after_next_enqueue();
   auto venus_peers = venus.ep.peers();
   CAF_REQUIRE_EQUAL(venus_peers.size(), 1u);
   CAF_CHECK_EQUAL(venus_peers.front().status, peer_status::peered);
-  MESSAGE("assume one peer for earth");
+  MESSAGE("assume two peers for earth");
   earth.loop_after_next_enqueue();
   auto earth_peers = earth.ep.peers();
   CAF_REQUIRE_EQUAL(earth_peers.size(), 1u);
@@ -313,29 +279,28 @@ CAF_TEST(topic_prefix_matching_async_subscribe) {
   MESSAGE("subscribe to 'zeek/events' on venus");
   venus.subscribe_to("zeek/events");
   MESSAGE("subscribe to 'zeek/events/failures' on earth");
-  earth.subscribe_to("zeek/events/failures");
+  earth.subscribe_to("zeek/events/errors");
   MESSAGE("verify subscriptions");
-  auto filter = [](std::initializer_list<topic> xs) -> std::vector<topic> {
-    return xs;
-  };
   mercury.loop_after_next_enqueue();
   CAF_CHECK_EQUAL(mercury.ep.peer_subscriptions(),
-                  filter({"zeek/events", "zeek/events/failures"}));
+                  filter_type({"zeek/events"}));
   venus.loop_after_next_enqueue();
-  CAF_CHECK_EQUAL(venus.ep.peer_subscriptions(), filter({}));
+  CAF_CHECK_EQUAL(venus.ep.peer_subscriptions(),
+                  filter_type({"zeek/events/errors"}));
   earth.loop_after_next_enqueue();
-  CAF_CHECK_EQUAL(earth.ep.peer_subscriptions(), filter({}));
-  MESSAGE("publish to 'zeek/events/(logging|failures)' on mercury");
-  mercury.publish("zeek/events/failures", "oops", "sorry!");
-  mercury.publish("zeek/events/logging", 123, 456);
+  CAF_CHECK_EQUAL(earth.ep.peer_subscriptions(),
+                  filter_type({"zeek/events"}));
+  MESSAGE("publish to 'zeek/events/(data|errors)' on mercury");
+  mercury.publish("zeek/events/errors", "oops", "sorry!");
+  mercury.publish("zeek/events/data", 123, 456);
   MESSAGE("verify published data");
   CAF_CHECK_EQUAL(mercury.data, data_msgs({}));
-  CAF_CHECK_EQUAL(venus.data, data_msgs({{"zeek/events/failures", "oops"},
-                                         {"zeek/events/failures", "sorry!"},
-                                         {"zeek/events/logging", 123},
-                                         {"zeek/events/logging", 456}}));
-  CAF_CHECK_EQUAL(earth.data, data_msgs({{"zeek/events/failures", "oops"},
-                                         {"zeek/events/failures", "sorry!"}}));
+  CAF_CHECK_EQUAL(venus.data, data_msgs({{"zeek/events/errors", "oops"},
+                                         {"zeek/events/errors", "sorry!"},
+                                         {"zeek/events/data", 123},
+                                         {"zeek/events/data", 456}}));
+  CAF_CHECK_EQUAL(earth.data, data_msgs({{"zeek/events/errors", "oops"},
+                                         {"zeek/events/errors", "sorry!"}}));
   venus.loop_after_next_enqueue();
   venus.ep.unpeer("mercury", 4040);
   earth.loop_after_next_enqueue();
@@ -352,12 +317,12 @@ CAF_TEST(topic_prefix_matching_make_subscriber) {
   CAF_REQUIRE_EQUAL(mercury_peers.size(), 2u);
   CAF_CHECK_EQUAL(mercury_peers.front().status, peer_status::peered);
   CAF_CHECK_EQUAL(mercury_peers.back().status, peer_status::peered);
-  MESSAGE("assume one peer for venus");
+  MESSAGE("assume two peers for venus");
   venus.loop_after_next_enqueue();
   auto venus_peers = venus.ep.peers();
   CAF_REQUIRE_EQUAL(venus_peers.size(), 1u);
   CAF_CHECK_EQUAL(venus_peers.front().status, peer_status::peered);
-  MESSAGE("assume one peer for earth");
+  MESSAGE("assume two peers for earth");
   earth.loop_after_next_enqueue();
   auto earth_peers = earth.ep.peers();
   CAF_REQUIRE_EQUAL(earth_peers.size(), 1u);
@@ -365,46 +330,40 @@ CAF_TEST(topic_prefix_matching_make_subscriber) {
   MESSAGE("subscribe to 'zeek/events' on venus");
   auto venus_s1 = venus.ep.make_subscriber({"zeek/events"});
   auto venus_s2 = venus.ep.make_subscriber({"zeek/events"});
-  venus_s1.set_rate_calculation(false);
-  venus_s2.set_rate_calculation(false);
   exec_loop();
-  MESSAGE("subscribe to 'zeek/events/failures' on earth");
-  auto earth_s1 = earth.ep.make_subscriber({"zeek/events/failures"});
-  auto earth_s2 = earth.ep.make_subscriber({"zeek/events/failures"});
-  earth_s1.set_rate_calculation(false);
-  earth_s2.set_rate_calculation(false);
+  MESSAGE("subscribe to 'zeek/events/errors' on earth");
+  auto earth_s1 = earth.ep.make_subscriber({"zeek/events/errors"});
+  auto earth_s2 = earth.ep.make_subscriber({"zeek/events/errors"});
   exec_loop();
   MESSAGE("verify subscriptions");
-  auto filter = [](std::initializer_list<topic> xs) -> std::vector<topic> {
-    return xs;
-  };
   mercury.loop_after_next_enqueue();
   CAF_CHECK_EQUAL(mercury.ep.peer_subscriptions(),
-                  filter({"zeek/events", "zeek/events/failures"}));
+                  filter_type({"zeek/events"}));
   venus.loop_after_next_enqueue();
-  CAF_CHECK_EQUAL(venus.ep.peer_subscriptions(), filter({}));
+  CAF_CHECK_EQUAL(venus.ep.peer_subscriptions(),
+                  filter_type({"zeek/events/errors"}));
   earth.loop_after_next_enqueue();
-  CAF_CHECK_EQUAL(earth.ep.peer_subscriptions(), filter({}));
-  MESSAGE("publish to 'zeek/events/(logging|failures)' on mercury");
-  mercury.publish("zeek/events/failures", "oops", "sorry!");
-  mercury.publish("zeek/events/logging", 123, 456);
+  CAF_CHECK_EQUAL(earth.ep.peer_subscriptions(), filter_type({"zeek/events"}));
+  MESSAGE("publish to 'zeek/events/(data|errors)' on mercury");
+  mercury.publish("zeek/events/errors", "oops", "sorry!");
+  mercury.publish("zeek/events/data", 123, 456);
   MESSAGE("verify published data");
   CAF_CHECK_EQUAL(venus_s1.poll(),
-                  data_msgs({{"zeek/events/failures", "oops"},
-                             {"zeek/events/failures", "sorry!"},
-                             {"zeek/events/logging", 123},
-                             {"zeek/events/logging", 456}}));
+                  data_msgs({{"zeek/events/errors", "oops"},
+                             {"zeek/events/errors", "sorry!"},
+                             {"zeek/events/data", 123},
+                             {"zeek/events/data", 456}}));
   CAF_CHECK_EQUAL(venus_s2.poll(),
-                  data_msgs({{"zeek/events/failures", "oops"},
-                             {"zeek/events/failures", "sorry!"},
-                             {"zeek/events/logging", 123},
-                             {"zeek/events/logging", 456}}));
+                  data_msgs({{"zeek/events/errors", "oops"},
+                             {"zeek/events/errors", "sorry!"},
+                             {"zeek/events/data", 123},
+                             {"zeek/events/data", 456}}));
   CAF_CHECK_EQUAL(earth_s1.poll(),
-                  data_msgs({{"zeek/events/failures", "oops"},
-                             {"zeek/events/failures", "sorry!"}}));
+                  data_msgs({{"zeek/events/errors", "oops"},
+                             {"zeek/events/errors", "sorry!"}}));
   CAF_CHECK_EQUAL(earth_s2.poll(),
-                  data_msgs({{"zeek/events/failures", "oops"},
-                             {"zeek/events/failures", "sorry!"}}));
+                  data_msgs({{"zeek/events/errors", "oops"},
+                             {"zeek/events/errors", "sorry!"}}));
   exec_loop();
   venus.loop_after_next_enqueue();
   venus.ep.unpeer("mercury", 4040);
@@ -414,8 +373,6 @@ CAF_TEST(topic_prefix_matching_make_subscriber) {
 
 // -- unpeering of nodes and emitted status/error messages ---------------------
 
-using event_value = status_subscriber::value_type;
-
 struct code {
   code(ec x) : value(x) {
     // nop
@@ -426,19 +383,18 @@ struct code {
   }
 
   code(const event_value& x) {
-    if (caf::holds_alternative<error>(x))
+    if (is<error>(x))
       value = static_cast<ec>(caf::get<error>(x).code());
     else
-      value = caf::get<status>(x).code();
+      value = get<status>(x).code();
   }
 
-  caf::variant<sc, ec> value;
+  std::variant<sc, ec> value;
 };
 
 std::string to_string(const code& x) {
-  return caf::holds_alternative<sc>(x.value)
-         ? to_string(caf::get<sc>(x.value))
-         : to_string(caf::get<ec>(x.value));
+  return is<sc>(x.value) ? to_string(std::get<sc>(x.value))
+                         : to_string(std::get<ec>(x.value));
 }
 
 bool operator==(const code& x, const code& y) {
@@ -449,11 +405,24 @@ std::vector<code> event_log(std::initializer_list<code> xs) {
   return {xs};
 }
 
-std::vector<code> event_log(const std::vector<event_value>& xs) {
+std::vector<code> event_log(const std::vector<event_value>& xs,
+                            bool make_unique = false) {
+  // For the purpose of this test, we only care about the peer_* statuses.
+  auto predicate = [](const auto& x) {
+    if constexpr (std::is_same<std::decay_t<decltype(x)>, status>::value) {
+      auto c = x.code();
+      return c == sc::peer_added || c == sc::peer_removed || c == sc::peer_lost;
+    } else {
+      return true;
+    }
+  };
   std::vector<code> ys;
   ys.reserve(xs.size());
   for (auto& x : xs)
-    ys.emplace_back(x);
+    if (std::visit(predicate, x))
+      ys.emplace_back(x);
+  if (make_unique)
+    ys.erase(std::unique(ys.begin(), ys.end()), ys.end());
   return ys;
 }
 
@@ -462,8 +431,6 @@ CAF_TEST(unpeering) {
   auto mercury_es = mercury.ep.make_status_subscriber(true);
   auto venus_es = venus.ep.make_status_subscriber(true);
   auto earth_es = earth.ep.make_status_subscriber(true);
-  for (auto es : {&mercury_es, &venus_es, &earth_es})
-    es->set_rate_calculation(false);
   exec_loop();
   connect_peers();
   CAF_CHECK_EQUAL(event_log(mercury_es.poll()),
@@ -506,7 +473,6 @@ CAF_TEST(unpeering) {
 CAF_TEST(unpeering_without_connections) {
   MESSAGE("get events from all peers");
   auto venus_es = venus.ep.make_status_subscriber(true);
-  venus_es.set_rate_calculation(false);
   exec_loop();
   MESSAGE("disconnect venus from non-existing peer");
   venus.loop_after_next_enqueue();
@@ -519,14 +485,12 @@ CAF_TEST(connection_retry) {
   MESSAGE("get events from mercury and venus");
   auto mercury_es = mercury.ep.make_status_subscriber(true);
   auto venus_es = venus.ep.make_status_subscriber(true);
-  for (auto es : {&mercury_es, &venus_es})
-    es->set_rate_calculation(false);
   exec_loop();
   MESSAGE("initiate peering from venus to mercury (will fail)");
   venus.ep.peer_nosync("mercury", 4040, std::chrono::seconds(1));
   MESSAGE("spawn helper that starts listening on mercury:4040 eventually");
   mercury.sys.spawn([&](caf::event_based_actor* self) -> caf::behavior {
-    self->delayed_send(self, std::chrono::seconds(2), broker::atom::ok_v);
+    self->delayed_send(self, std::chrono::seconds(2), atom::ok_v);
     return {
       [&](caf::ok_atom) {
         MESSAGE("start listening on mercury:4040");
@@ -548,7 +512,7 @@ CAF_TEST(connection_retry) {
   exec_loop();
   MESSAGE("check event logs");
   CAF_CHECK_EQUAL(event_log(mercury_es.poll()), event_log({sc::peer_added}));
-  CAF_CHECK_EQUAL(event_log(venus_es.poll()),
+  CAF_CHECK_EQUAL(event_log(venus_es.poll(), true),
                   event_log({ec::peer_unavailable, sc::peer_added}));
   MESSAGE("disconnect venus from mercury");
   venus.loop_after_next_enqueue();
