@@ -16,6 +16,7 @@
  */
 #include <err.h>
 #include <signal.h>
+#include <inttypes.h>
 #include <sys/prctl.h>
 #include <sys/wait.h>
 #include <unistd.h>
@@ -36,17 +37,34 @@ struct proc_pid {
     LIST_HEAD(pid_waiters, knote) ppd_proc_waiters; //!< knotes that are waiting on this PID.
 };
 
-static pthread_mutex_t     proc_init_mtx = PTHREAD_MUTEX_INITIALIZER;
-static unsigned int        proc_count = 0;
-static pthread_t           proc_wait_thread_id;
+/** Synchronisation for wait thread information
+ *
+ * This ensures that the number of process being monitored is correct
+ * and stops multiple threads attempting to start a monitor thread
+ * at the same time.
+ */
+static tracing_mutex_t     proc_init_mtx = TRACING_MUTEX_INITIALIZER;
+static int                 proc_count = 0;
+static pthread_t           proc_wait_thread;
+static pid_t               proc_wait_tid; /* Monitoring thread */
+
+/** Synchronisation for thread start
+ *
+ * This prevents any threads from continuing whilst the wait thread
+ * is started.
+ */
+static pthread_mutex_t     proc_wait_thread_mtx = PTHREAD_MUTEX_INITIALIZER;
+static pthread_cond_t      proc_wait_thread_cond = PTHREAD_COND_INITIALIZER;
 
 /** The global PID tree
  *
  * Contains all the PIDs any kqueue is interested in waiting on
  */
 static RB_HEAD(pid_index, proc_pid) proc_pid_index;
-static pthread_mutex_t              proc_pid_index_mtx = PTHREAD_MUTEX_INITIALIZER;
-
+static tracing_mutex_t    proc_pid_index_mtx = TRACING_MUTEX_INITIALIZER;
+#ifndef _WIN32
+pthread_mutexattr_t       proc_pid_index_mtx_attr;
+#endif
 
 static int
 proc_pid_cmp(struct proc_pid *a, struct proc_pid *b)
@@ -84,12 +102,34 @@ waiter_notify(struct proc_pid *ppd, int status)
         filt = knote_get_filter(kn);
         dbg_printf("pid=%u exited, notifying kq=%u filter=%p kn=%p",
                    (unsigned int)ppd->ppd_pid, kn->kn_kq->kq_id, filt, kn);
-        pthread_mutex_lock(&filt->kf_knote_mtx);
         kqops.eventfd_raise(&filt->kf_proc_eventfd);
-        LIST_INSERT_HEAD(&filt->kf_ready, kn, kn_ready);
-        pthread_mutex_unlock(&filt->kf_knote_mtx);
+        LIST_INSERT_HEAD(&filt->kf_ready, kn, kn_ready); /* protected by proc_pid_index_mtx */
 
-        LIST_REMOVE(kn, kn_proc_waiter);
+        LIST_REMOVE_ZERO(kn, kn_proc_waiter);
+    }
+
+    dbg_printf("pid=%u removing waiter list", (unsigned int)ppd->ppd_pid);
+    RB_REMOVE(pid_index, &proc_pid_index, ppd);
+    free(ppd);
+}
+
+static void
+waiter_notify_error(struct proc_pid *ppd, int wait_errno)
+{
+    struct knote *kn;
+    struct filter *filt;
+
+    while ((kn = LIST_FIRST(&ppd->ppd_proc_waiters))) {
+        kn->kev.flags |= EV_ERROR;
+        kn->kev.data = wait_errno;
+
+        filt = knote_get_filter(kn);
+        dbg_printf("pid=%u errored (%s), notifying kq=%u filter=%p kn=%p",
+                   (unsigned int)ppd->ppd_pid, strerror(errno), kn->kn_kq->kq_id, filt, kn);
+        kqops.eventfd_raise(&filt->kf_proc_eventfd);
+        LIST_INSERT_HEAD(&filt->kf_ready, kn, kn_ready); /* protected by proc_pid_index_mtx */
+
+        LIST_REMOVE_ZERO(kn, kn_proc_waiter);
     }
 
     dbg_printf("pid=%u removing waiter list", (unsigned int)ppd->ppd_pid);
@@ -136,31 +176,21 @@ waiter_siginfo_to_status(siginfo_t *info)
  *
  */
 static void *
-wait_thread(UNUSED void *arg)
+wait_thread_loop(UNUSED void *arg)
 {
     int status;
-    int ret;
+    int ret = 0;
     siginfo_t info;
     sigset_t sigmask;
     struct proc_pid *ppd, *ppd_tmp;
 
-    /* Block all signals except SIGCHLD */
-    sigfillset(&sigmask);
-    sigdelset(&sigmask, SIGCHLD);
-    pthread_sigmask(SIG_BLOCK, &sigmask, NULL);
-
     /* Set the thread's name to something descriptive so it shows up in gdb,
      * etc. Max name length is 16 bytes. */
     prctl(PR_SET_NAME, "libkqueue_wait", 0, 0, 0);
 
-    /*
-     * Only listen on SIGCHLD
-     */
-    sigemptyset(&sigmask);
-    sigaddset(&sigmask, SIGCHLD);
-    pthread_sigmask(SIG_UNBLOCK, &sigmask, NULL);
+    proc_wait_tid = syscall(SYS_gettid);
 
-    dbg_printf("started and waiting for SIGCHLD");
+    dbg_printf("tid=%u - waiter thread started", proc_wait_tid);
 
     /*
      * Native kqueue implementations leave processes monitored
@@ -232,22 +262,66 @@ wait_thread(UNUSED void *arg)
      * a native solution for Solaris this POSIX EVFILT_PROC
      * code must remain to provide a fallback mechanism.
      */
-    while ((ret = sigwaitinfo(&sigmask, &info))) {
+
+    /*
+     * Block all signals - sigwaitinfo isn't affected by this
+     */
+    sigfillset(&sigmask);
+    pthread_sigmask(SIG_BLOCK, &sigmask, NULL);
+
+    /*
+     * Only listen on SIGCHLD wait sigwaitinfo
+     */
+    sigemptyset(&sigmask);
+    sigaddset(&sigmask, SIGCHLD);
+
+    /*
+     * Inform the parent that we started correctly
+     */
+    pthread_mutex_lock(&proc_wait_thread_mtx);    /* Must try to lock to ensure parent is waiting on signal */
+    pthread_cond_signal(&proc_wait_thread_cond);
+    pthread_mutex_unlock(&proc_wait_thread_mtx);
+    do {
+        /*
+         * Don't allow the thread to be cancelled until we've
+         * finished one loop in the monitoring thread.  This
+         * ensures there are no nasty issue on exit.
+         */
+        pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
         if (ret < 0) {
             dbg_printf("sigwaitinfo(2): %s", strerror(errno));
             continue;
         }
 
-        dbg_printf("received SIGCHLD");
-
-        pthread_mutex_lock(&proc_pid_index_mtx);
         /*
-         * Check if this is a process we want to monitor
+         * The knote_* functions (i.e. those that could free a
+         * knote).
+         *
+         * All attempt to take this lock before modifying the
+         * knotes, wo we shouldn't have any issues here with
+         * knotes being freed out from underneath the waiter
+         * thread.
          */
-        ppd = RB_FIND(pid_index, &proc_pid_index, &(struct proc_pid){ .ppd_pid = info.si_pid });
-        if (ppd) {
-            status = waiter_siginfo_to_status(&info);
-            if (status >= 0) waiter_notify(ppd, status);  /* If < 0 notification is spurious */
+        tracing_mutex_lock(&proc_pid_index_mtx);
+
+        /*
+         * Sig 0 is the NULL signal.  This means it's the first
+         * iteration through the loop, and we just want to scan
+         * any existing PIDs.
+         *
+         * This fixes a potential race between the thread
+         * starting, EVFILT_PROC kevents being added, and a
+         * process exiting.
+         */
+        if (ret > 0) {
+            /*
+             * Check if this is a process we want to monitor
+             */
+            ppd = RB_FIND(pid_index, &proc_pid_index, &(struct proc_pid){ .ppd_pid = info.si_pid });
+            if (ppd) {
+                status = waiter_siginfo_to_status(&info);
+                if (status >= 0) waiter_notify(ppd, status);  /* If < 0 notification is spurious */
+            }
         }
 
         /*
@@ -260,7 +334,9 @@ wait_thread(UNUSED void *arg)
                 switch (errno) {
                 case ECHILD:
                     dbg_printf("waitid(2): pid=%u reaped too early - %s", ppd->ppd_pid, strerror(errno));
-                    continue; /* FIXME - Maybe produce an EV_ERROR for each of the knotes? */
+
+                    waiter_notify_error(ppd, errno);
+                    continue;
 
                 case EINTR:
                     goto again;
@@ -270,14 +346,45 @@ wait_thread(UNUSED void *arg)
             status = waiter_siginfo_to_status(&info);
             if (status >= 0) waiter_notify(ppd, status);  /* If < 0 notification is spurious */
         }
-        pthread_mutex_unlock(&proc_pid_index_mtx);
-    }
+        tracing_mutex_unlock(&proc_pid_index_mtx);
+
+        dbg_printf("waiting for SIGCHLD");
+        pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL); /* sigwaitinfo is the next cancellation point */
+    } while ((ret = sigwaitinfo(&sigmask, &info)));
 
     dbg_printf("exited");
 
     return (NULL);
 }
 
+static void
+evfilt_proc_libkqueue_init(void)
+{
+    /*
+     * Initialise the global PID index tree.  This needs to be
+     * recursive as the delete/enable/disable functions all
+     * need to lock the mutex, and they may be called indirectly
+     * in the copyout function (which already locks this mutex),
+     * as well as by other kqueue code.
+     */
+#ifndef _WIN32
+     pthread_mutexattr_init(&proc_pid_index_mtx_attr);
+     pthread_mutexattr_settype(&proc_pid_index_mtx_attr, PTHREAD_MUTEX_RECURSIVE);
+#endif
+     tracing_mutex_init(&proc_pid_index_mtx, &proc_pid_index_mtx_attr);
+}
+
+static void
+evfilt_proc_libkqueue_fork(void)
+{
+    /*
+     * The wait thread isn't duplicated in the forked copy
+     * of the process, and we need to prevent a cancellation
+     * request being sent to it when all the knotes are freed.
+     */
+    proc_wait_tid = 0;
+}
+
 static int
 evfilt_proc_init(struct filter *filt)
 {
@@ -295,7 +402,7 @@ evfilt_proc_init(struct filter *filt)
     /*
      * Initialise global resources (wait thread and PID tree).
      */
-    pthread_mutex_lock(&proc_init_mtx);
+    tracing_mutex_lock(&proc_init_mtx);
     if (proc_count == 0) {
         sigset_t sigmask;
 
@@ -314,13 +421,19 @@ evfilt_proc_init(struct filter *filt)
 
         dbg_printf("creating wait thread");
 
-        if (pthread_create(&proc_wait_thread_id, NULL, wait_thread, NULL) != 0) {
-            pthread_mutex_unlock(&proc_init_mtx);
+        pthread_mutex_lock(&proc_wait_thread_mtx);
+        if (pthread_create(&proc_wait_thread, NULL, wait_thread_loop, NULL) != 0) {
+            tracing_mutex_unlock(&proc_init_mtx);
+            pthread_mutex_unlock(&proc_wait_thread_mtx);
+
             goto error_1;
         }
-        proc_count++;
+        /* Wait until the wait thread has started and is monitoring signals */
+        pthread_cond_wait(&proc_wait_thread_cond, &proc_wait_thread_mtx);
+        pthread_mutex_unlock(&proc_wait_thread_mtx);
     }
-    pthread_mutex_unlock(&proc_init_mtx);
+    proc_count++;
+    tracing_mutex_unlock(&proc_init_mtx);
 
     return (0);
 }
@@ -332,11 +445,37 @@ evfilt_proc_destroy(struct filter *filt)
      * Free global resources like the wait thread
      * and PID tree.
      */
-    pthread_mutex_lock(&proc_init_mtx);
-    if (--proc_count == 0) {
-        pthread_cancel(proc_wait_thread_id);
+    tracing_mutex_lock(&proc_init_mtx);
+    assert(proc_count > 0);
+    /*
+     * Only cancel the wait thread if we're not
+     * in a forked copy as fork does not produce
+     * a new copy of the thread.
+     */
+    if ((--proc_count == 0) && (proc_wait_tid != 0)) {
+#ifndef NDEBUG
+        pid_t tid = proc_wait_tid;
+#endif
+        void *retval;
+        int ret;
+
+        dbg_printf("tid=%u - cancelling", tid);
+        ret = pthread_cancel(proc_wait_thread);
+        if (ret != 0)
+           dbg_printf("tid=%u - cancel failed: %s", tid, strerror(ret));
+
+        ret = pthread_join(proc_wait_thread, &retval);
+        if (ret == 0) {
+            if (retval == PTHREAD_CANCELED) {
+                dbg_printf("tid=%u - joined with exit_status=PTHREAD_CANCELED", tid);
+            } else {
+                dbg_printf("tid=%u - joined with exit_status=%" PRIdPTR, tid, (intptr_t)retval);
+            }
+        } else {
+            dbg_printf("tid=%u - join failed: %s", tid, strerror(ret));
+        }
     }
-    pthread_mutex_unlock(&proc_init_mtx);
+    tracing_mutex_unlock(&proc_init_mtx);
 
     kqops.eventfd_unregister(filt->kf_kqueue, &filt->kf_proc_eventfd);
     kqops.eventfd_close(&filt->kf_proc_eventfd);
@@ -347,20 +486,25 @@ evfilt_proc_knote_create(struct filter *filt, struct knote *kn)
 {
     struct proc_pid *ppd;
 
-    pthread_mutex_lock(&proc_pid_index_mtx);
+    tracing_mutex_lock(&proc_pid_index_mtx);
+    /*
+     * Fixme - We should probably check to see if the PID exists
+     * here and error out early instead of waiting for the waiter
+     * loop to tell us.
+     */
     ppd = RB_FIND(pid_index, &proc_pid_index, &(struct proc_pid){ .ppd_pid = kn->kev.ident });
     if (!ppd) {
         dbg_printf("pid=%u adding waiter list", (unsigned int)kn->kev.ident);
         ppd = calloc(1, sizeof(struct proc_pid));
         if (unlikely(!ppd)) {
-            pthread_mutex_unlock(&proc_pid_index_mtx);
+            tracing_mutex_unlock(&proc_pid_index_mtx);
             return -1;
         }
         ppd->ppd_pid = kn->kev.ident;
         RB_INSERT(pid_index, &proc_pid_index, ppd);
     }
     LIST_INSERT_HEAD(&ppd->ppd_proc_waiters, kn, kn_proc_waiter);
-    pthread_mutex_unlock(&proc_pid_index_mtx);
+    tracing_mutex_unlock(&proc_pid_index_mtx);
 
     /*
      * These get added by default on macOS (and likely FreeBSD)
@@ -385,8 +529,8 @@ evfilt_proc_knote_delete(UNUSED struct filter *filt, struct knote *kn)
 {
     struct proc_pid *ppd;
 
-    pthread_mutex_lock(&proc_pid_index_mtx);
-    if (LIST_INSERTED(kn, kn_proc_waiter)) LIST_REMOVE(kn, kn_proc_waiter);
+    tracing_mutex_lock(&proc_pid_index_mtx);
+    if (LIST_INSERTED(kn, kn_proc_waiter)) LIST_REMOVE_ZERO(kn, kn_proc_waiter);
 
     /*
      * ppd may have been removed already if there
@@ -404,7 +548,7 @@ evfilt_proc_knote_delete(UNUSED struct filter *filt, struct knote *kn)
     } else {
         dbg_printf("pid=%u waiter list already removed", (unsigned int)kn->kev.ident);
     }
-    pthread_mutex_unlock(&proc_pid_index_mtx);
+    tracing_mutex_unlock(&proc_pid_index_mtx);
 
     return (0);
 }
@@ -422,9 +566,9 @@ evfilt_proc_knote_disable(UNUSED struct filter *filt, struct knote *kn)
      * which is why we need to run the same logic
      * as knote_create when re-enabling.
      */
-    pthread_mutex_lock(&proc_pid_index_mtx);
-    if (LIST_INSERTED(kn, kn_proc_waiter)) LIST_REMOVE(kn, kn_proc_waiter);
-    pthread_mutex_unlock(&proc_pid_index_mtx);
+    tracing_mutex_lock(&proc_pid_index_mtx);
+    if (LIST_INSERTED(kn, kn_proc_waiter)) LIST_REMOVE_ZERO(kn, kn_proc_waiter);
+    tracing_mutex_unlock(&proc_pid_index_mtx);
 
     return (0);
 }
@@ -433,54 +577,77 @@ static int
 evfilt_proc_knote_copyout(struct kevent *dst, int nevents, struct filter *filt,
     struct knote *kn, void *ev)
 {
-    struct knote *tmp;
-    int events = 0;
+    struct kevent *dst_p = dst, *dst_end = dst_p + nevents;
 
     /*
      * Prevent the waiter thread from modifying
      * the knotes in the ready list whilst we're
      * processing them.
+     *
+     * At first glance this wouldn't appear to
+     * require a global lock.  The issue is that
+     * if we use filter-specific locks to protect
+     * the kf_ready list, we can get into a deadlock
+     * where the waiter thread holds the lock on
+     * proc_pid_index_mtx, and it attempting to lock
+     * the filter lock to insert new ready
+     * notifications, and we're holding the filter
+     * lock, and attempting to delete a knote,
+     * which requires holding the proc_pid_index_mtx.
+     *
+     * This was seen in the real world and caused
+     * major issues.
+     *
+     * By using a single lock we ensure either the
+     * waiter thread is notifying kqueue instances
+     * data is ready, or a kqueue is copying out
+     * notifications, so no deadlock can occur.
      */
-    pthread_mutex_lock(&filt->kf_knote_mtx);
+    tracing_mutex_lock(&proc_pid_index_mtx);
+    assert(!LIST_EMPTY(&filt->kf_ready));
 
     /*
      * kn arg is always NULL here, so we just reuse it
      * for the loop.
      */
-    LIST_FOREACH_SAFE(kn, &filt->kf_ready, kn_ready, tmp) {
-        if (++events > nevents)
+    while ((kn = LIST_FIRST(&filt->kf_ready))) {
+        if (dst_p >= dst_end)
             break;
 
         kevent_dump(&kn->kev);
-        memcpy(dst, &kn->kev, sizeof(*dst));
-        dst->fflags = NOTE_EXIT;
-        dst->flags |= EV_EOF;
-        dst->data = kn->kn_proc_status;
+        memcpy(dst_p, &kn->kev, sizeof(*dst));
+        dst_p->fflags = NOTE_EXIT;
+        dst_p->flags |= EV_EOF;
+        dst_p->data = kn->kn_proc_status;
+
+        LIST_REMOVE_ZERO(kn, kn_ready); /* knote_copyout_flag_actions may free the knote */
 
         if (knote_copyout_flag_actions(filt, kn) < 0) {
-            pthread_mutex_unlock(&filt->kf_knote_mtx);
+            LIST_INSERT_HEAD(&filt->kf_ready, kn, kn_ready);
+            tracing_mutex_unlock(&proc_pid_index_mtx);
             return -1;
         }
 
-        dst++;
+        dst_p++;
     }
 
     if (LIST_EMPTY(&filt->kf_ready))
         kqops.eventfd_lower(&filt->kf_proc_eventfd);
+    tracing_mutex_unlock(&proc_pid_index_mtx);
 
-    pthread_mutex_unlock(&filt->kf_knote_mtx);
-
-    return (nevents);
+    return (dst_p - dst);
 }
 
 const struct filter evfilt_proc = {
-    .kf_id      = EVFILT_PROC,
-    .kf_init    = evfilt_proc_init,
-    .kf_destroy = evfilt_proc_destroy,
-    .kf_copyout = evfilt_proc_knote_copyout,
-    .kn_create  = evfilt_proc_knote_create,
-    .kn_modify  = evfilt_proc_knote_modify,
-    .kn_enable  = evfilt_proc_knote_create,
-    .kn_disable = evfilt_proc_knote_disable,
-    .kn_delete  = evfilt_proc_knote_delete
+    .libkqueue_init   = evfilt_proc_libkqueue_init,
+    .libkqueue_fork   = evfilt_proc_libkqueue_fork,
+    .kf_id            = EVFILT_PROC,
+    .kf_init          = evfilt_proc_init,
+    .kf_destroy       = evfilt_proc_destroy,
+    .kf_copyout       = evfilt_proc_knote_copyout,
+    .kn_create        = evfilt_proc_knote_create,
+    .kn_modify        = evfilt_proc_knote_modify,
+    .kn_enable        = evfilt_proc_knote_create,
+    .kn_disable       = evfilt_proc_knote_disable,
+    .kn_delete        = evfilt_proc_knote_delete
 };
