@@ -13,6 +13,7 @@
 #include <fstream>
 
 #include "zeek/3rdparty/doctest.h"
+#include "zeek/Obj.h"
 #include "zeek/Reporter.h"
 #include "zeek/util.h"
 
@@ -634,14 +635,15 @@ int Dictionary::Capacity(bool expected) const
 
 int Dictionary::ThresholdEntries() const
 	{
-	// Increase the size of the dictionary when it is 75% full. However, when the dictionary
-	// is small ( <= 20 elements ), only resize it when it's 100% full. The dictionary will
-	// always resize when the current insertion causes it to be full. This ensures that the
-	// current insertion should always be successful.
+	// Increase the size of the dictionary when it is 75% full. However, when
+	// the dictionary is small ( bucket_capacity <= 2^3+3=11 elements ), only
+	// resize it when it's 100% full. The dictionary will always resize when the
+	// current insertion causes it to be full. This ensures that the current
+	// insertion should always be successful.
 	int capacity = Capacity();
 	if ( log2_buckets <= detail::DICT_THRESHOLD_BITS )
-		return capacity; // 20 or less elements, 1.0, only size up when necessary.
-	return capacity - (capacity >> detail::DICT_LOAD_FACTOR_BITS);
+		return capacity;
+	return capacity * detail::DICT_LOAD_FACTOR_100 / 100;
 	}
 
 detail::hash_t Dictionary::FibHash(detail::hash_t h) const
@@ -1136,8 +1138,8 @@ int Dictionary::LookupIndex(const void* key, int key_size, detail::hash_t hash,
 // Insert
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-void* Dictionary::Insert(void* key, int key_size, detail::hash_t hash, void* val, bool copy_key,
-                         bool* iterators_invalidated)
+void* Dictionary::Insert(void* key, uint64_t key_size, detail::hash_t hash, void* val,
+                         bool copy_key, bool* iterators_invalidated)
 	{
 	ASSERT_VALID(this);
 
@@ -1148,6 +1150,21 @@ void* Dictionary::Insert(void* key, int key_size, detail::hash_t hash, void* val
 
 	void* v = nullptr;
 
+	if ( key_size > detail::DictEntry::MAX_KEY_SIZE )
+		{
+		// If the key is bigger than something that will fit in a DictEntry, report a
+		// RuntimeError. This will throw an exception. If this call came from a script
+		// context, it'll cause the script interpreter to unwind and stop the script
+		// execution. If called elsewhere, Zeek will likely abort due to an unhandled
+		// exception. This is all entirely intentional. since if you got to this point
+		// something went really wrong with your input data.
+		auto loc = detail::GetCurrentLocation();
+		reporter->RuntimeError(&loc,
+		                       "Attempted to create DictEntry with excessively large key, "
+		                       "truncating key (%" PRIu64 " > %d)",
+		                       key_size, detail::DictEntry::MAX_KEY_SIZE);
+		}
+
 	// Look to see if this key is already in the table. If found, insert_position is the
 	// position of the existing element. If not, insert_position is where it'll be inserted
 	// and insert_distance is the distance of the key for the position.
@@ -1217,6 +1234,13 @@ void* Dictionary::Insert(void* key, int key_size, detail::hash_t hash, void* val
 			max_entries = num_entries;
 		if ( num_entries > ThresholdEntries() )
 			SizeUp();
+		// if space_distance is too great, performance decreases. we need to sizeup for
+		// performance.
+		else if ( space_distance_samples > detail::MIN_SPACE_DISTANCE_SAMPLES &&
+		          space_distance_sum >
+		              uint64_t(space_distance_samples) * detail::SPACE_DISTANCE_THRESHOLD &&
+		          int(num_entries) > detail::MIN_DICT_LOAD_FACTOR_100 * Capacity() / 100 )
+			SizeUp();
 		}
 
 	// Remap after insert can adjust asap to shorten period of mixed table.
@@ -1231,9 +1255,9 @@ void* Dictionary::Insert(void* key, int key_size, detail::hash_t hash, void* val
 /// e.distance is adjusted to be the one at insert_position.
 void Dictionary::InsertRelocateAndAdjust(detail::DictEntry& entry, int insert_position)
 	{
-#ifdef ZEEK_DICT_DEBUG
+#ifdef DEBUG
 	entry.bucket = BucketByHash(entry.hash, log2_buckets);
-#endif // ZEEK_DICT_DEBUG
+#endif // DEBUG
 	int last_affected_position = insert_position;
 	InsertAndRelocate(entry, insert_position, &last_affected_position);
 
@@ -1358,9 +1382,13 @@ void Dictionary::SizeUp()
 
 	// another remap starts.
 	remaps++; // used in Lookup() to cover SizeUp with incomplete remaps.
-	ASSERT(
-		remaps <=
-		log2_buckets); // because we only sizeUp, one direction. we know the previous log2_buckets.
+
+	// because we only sizeUp, one direction. we know the previous log2_buckets.
+	ASSERT(remaps <= log2_buckets);
+
+	// reset performance metrics.
+	space_distance_sum = 0;
+	space_distance_samples = 0;
 	}
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -1554,9 +1582,9 @@ bool Dictionary::Remap(int position, int* new_position)
 		return false;
 	detail::DictEntry entry = RemoveAndRelocate(
 		position); // no iteration cookies to adjust, no need for last_affected_position.
-#ifdef ZEEK_DICT_DEBUG
+#ifdef DEBUG
 	entry.bucket = expected;
-#endif // ZEEK_DICT_DEBUG
+#endif // DEBUG
 
 	// find insert position.
 	int insert_position = EndOfClusterByBucket(expected);
