@@ -23,21 +23,19 @@ using namespace zeek::detail::trigger;
 namespace zeek::detail::trigger
 	{
 
+// Used to extract the globals and locals seen in a trigger expression.
 class TriggerTraversalCallback : public TraversalCallback
 	{
 public:
-	TriggerTraversalCallback(Trigger* arg_trigger)
+	TriggerTraversalCallback(IDSet& _globals, IDSet& _locals) : globals(_globals), locals(_locals)
 		{
-		Ref(arg_trigger);
-		trigger = arg_trigger;
 		}
 
-	~TriggerTraversalCallback() { Unref(trigger); }
-
 	virtual TraversalCode PreExpr(const Expr*) override;
 
 private:
-	Trigger* trigger;
+	IDSet& globals;
+	IDSet& locals;
 	};
 
 TraversalCode trigger::TriggerTraversalCallback::PreExpr(const Expr* expr)
@@ -50,14 +48,12 @@ TraversalCode trigger::TriggerTraversalCallback::PreExpr(const Expr* expr)
 		case EXPR_NAME:
 			{
 			const auto* e = static_cast<const NameExpr*>(expr);
-			if ( e->Id()->IsGlobal() )
-				trigger->Register(e->Id());
+			auto id = e->Id();
 
-			Val* v = e->Id()->GetVal().get();
-
-			if ( v && v->Modifiable() )
-				trigger->Register(v);
-			break;
+			if ( id->IsGlobal() )
+				globals.insert(id);
+			else
+				locals.insert(id);
 			};
 
 		default:
@@ -103,54 +99,49 @@ protected:
 	double time;
 	};
 
-Trigger::Trigger(const Expr* cond, Stmt* body, Stmt* timeout_stmts, Expr* timeout_expr,
-                 Frame* frame, bool is_return, const Location* location)
-	{
-	timeout_value = -1;
-
-	if ( timeout_expr )
-		{
-		ValPtr timeout_val;
-
-		try
-			{
-			timeout_val = timeout_expr->Eval(frame);
-			}
-		catch ( InterpreterException& )
-			{ /* Already reported */
-			}
-
-		if ( timeout_val )
-			timeout_value = timeout_val->AsInterval();
-		}
-
-	Init(cond, body, timeout_stmts, frame, is_return, location);
-	}
-
-Trigger::Trigger(const Expr* cond, Stmt* body, Stmt* timeout_stmts, double timeout, Frame* frame,
+Trigger::Trigger(ExprPtr cond, StmtPtr body, StmtPtr timeout_stmts, double timeout, Frame* frame,
                  bool is_return, const Location* location)
 	{
 	timeout_value = timeout;
 	Init(cond, body, timeout_stmts, frame, is_return, location);
 	}
 
-void Trigger::Init(const Expr* arg_cond, Stmt* arg_body, Stmt* arg_timeout_stmts, Frame* arg_frame,
-                   bool arg_is_return, const Location* arg_location)
+Trigger::Trigger(WhenInfo* wi, double timeout, const IDSet& _globals,
+                 std::vector<ValPtr> _local_aggrs, Frame* f, const Location* loc)
+	{
+	timeout_value = timeout;
+	globals = _globals;
+	local_aggrs = std::move(_local_aggrs);
+	have_trigger_elems = true;
+
+	Init(wi->Cond(), wi->WhenBody(), wi->TimeoutStmt(), f, wi->IsReturn(), loc);
+	}
+
+void Trigger::Init(ExprPtr arg_cond, StmtPtr arg_body, StmtPtr arg_timeout_stmts, Frame* arg_frame,
+                   bool arg_is_return, const Location* location)
 	{
 	cond = arg_cond;
 	body = arg_body;
 	timeout_stmts = arg_timeout_stmts;
-	frame = arg_frame->Clone();
 	timer = nullptr;
 	delayed = false;
 	disabled = false;
 	attached = nullptr;
 	is_return = arg_is_return;
-	location = arg_location;
+
+	if ( location )
+		name = util::fmt("%s:%d-%d", location->filename, location->first_line, location->last_line);
+	else
+		name = "<no-trigger-location>";
+
+	if ( arg_frame )
+		frame = arg_frame->Clone();
+	else
+		frame = nullptr;
 
 	DBG_LOG(DBG_NOTIFIERS, "%s: instantiating", Name());
 
-	if ( is_return )
+	if ( is_return && frame && arg_frame )
 		{
 		Trigger* parent = frame->GetTrigger();
 		if ( ! parent )
@@ -217,8 +208,30 @@ void Trigger::ReInit(std::vector<ValPtr> index_expr_results)
 	{
 	assert(! disabled);
 	UnregisterAll();
-	TriggerTraversalCallback cb(this);
-	cond->Traverse(&cb);
+
+	if ( ! have_trigger_elems )
+		{
+		TriggerTraversalCallback cb(globals, locals);
+		cond->Traverse(&cb);
+		have_trigger_elems = true;
+		}
+
+	for ( auto g : globals )
+		{
+		Register(g);
+
+		auto& v = g->GetVal();
+		if ( v && v->Modifiable() )
+			Register(v.get());
+		}
+
+	for ( auto l : locals )
+		{
+		ASSERT(! l->GetVal());
+		}
+
+	for ( auto& av : local_aggrs )
+		Register(av.get());
 
 	for ( const auto& v : index_expr_results )
 		Register(v.get());
@@ -312,7 +325,7 @@ bool Trigger::Eval()
 		{
 		Trigger* trigger = frame->GetTrigger();
 		assert(trigger);
-		assert(frame->GetCall());
+		assert(frame->GetTriggerAssoc());
 		assert(trigger->attached == this);
 
 #ifdef DEBUG
@@ -321,7 +334,7 @@ bool Trigger::Eval()
 		delete[] pname;
 #endif
 
-		auto queued = trigger->Cache(frame->GetCall(), v.get());
+		auto queued = trigger->Cache(frame->GetTriggerAssoc(), v.get());
 		trigger->Release();
 		frame->ClearTrigger();
 
@@ -366,7 +379,7 @@ void Trigger::Timeout()
 			{
 			Trigger* trigger = frame->GetTrigger();
 			assert(trigger);
-			assert(frame->GetCall());
+			assert(frame->GetTriggerAssoc());
 			assert(trigger->attached == this);
 
 #ifdef DEBUG
@@ -375,7 +388,7 @@ void Trigger::Timeout()
 			        pname);
 			delete[] pname;
 #endif
-			auto queued = trigger->Cache(frame->GetCall(), v.get());
+			auto queued = trigger->Cache(frame->GetTriggerAssoc(), v.get());
 			trigger->Release();
 			frame->ClearTrigger();
 
@@ -390,9 +403,10 @@ void Trigger::Timeout()
 	Unref(this);
 	}
 
-void Trigger::Register(ID* id)
+void Trigger::Register(const ID* const_id)
 	{
 	assert(! disabled);
+	ID* id = const_cast<ID*>(const_id);
 	notifier::detail::registry.Register(id, this);
 
 	Ref(id);
@@ -441,12 +455,12 @@ void Trigger::Attach(Trigger* trigger)
 	Hold();
 	}
 
-bool Trigger::Cache(const CallExpr* expr, Val* v)
+bool Trigger::Cache(const void* obj, Val* v)
 	{
 	if ( disabled || ! v )
 		return false;
 
-	ValCache::iterator i = cache.find(expr);
+	ValCache::iterator i = cache.find(obj);
 
 	if ( i != cache.end() )
 		{
@@ -455,7 +469,7 @@ bool Trigger::Cache(const CallExpr* expr, Val* v)
 		}
 
 	else
-		cache.insert(ValCache::value_type(expr, v));
+		cache.insert(ValCache::value_type(obj, v));
 
 	Ref(v);
 
@@ -463,11 +477,11 @@ bool Trigger::Cache(const CallExpr* expr, Val* v)
 	return true;
 	}
 
-Val* Trigger::Lookup(const CallExpr* expr)
+Val* Trigger::Lookup(const void* obj)
 	{
 	assert(! disabled);
 
-	ValCache::iterator i = cache.find(expr);
+	ValCache::iterator i = cache.find(obj);
 	return (i != cache.end()) ? i->second : 0;
 	}
 
@@ -487,12 +501,6 @@ void Trigger::Modified(notifier::detail::Modifiable* m)
 	trigger_mgr->Queue(this);
 	}
 
-const char* Trigger::Name() const
-	{
-	assert(location);
-	return util::fmt("%s:%d-%d", location->filename, location->first_line, location->last_line);
-	}
-
 Manager::Manager() : iosource::IOSource()
 	{
 	pending = new TriggerList();
