@@ -420,8 +420,9 @@ static void BuildJSON(threading::formatter::JSON::NullDoubleWriter& writer, Val*
 		}
 
 	rapidjson::Value j;
+	auto tag = val->GetType()->Tag();
 
-	switch ( val->GetType()->Tag() )
+	switch ( tag )
 		{
 		case TYPE_BOOL:
 			writer.Bool(val->AsBool());
@@ -475,8 +476,15 @@ static void BuildJSON(threading::formatter::JSON::NullDoubleWriter& writer, Val*
 			ODesc d;
 			d.SetStyle(RAW_STYLE);
 			val->Describe(&d);
-			writer.String(util::json_escape_utf8(
-				std::string(reinterpret_cast<const char*>(d.Bytes()), d.Len())));
+			std::string desc(reinterpret_cast<const char*>(d.Bytes()), d.Len());
+
+			// None of our function types should have surrounding
+			// whitespace, but ODesc might produce it due to its
+			// many output modes and flags. Strip it.
+			if ( tag == TYPE_FUNC )
+				desc = util::strstrip(desc);
+
+			writer.String(util::json_escape_utf8(desc));
 			break;
 			}
 
@@ -939,9 +947,7 @@ StringVal::StringVal(int length, const char* s)
 	{
 	}
 
-StringVal::StringVal(const char* s) : StringVal(new String(s)) { }
-
-StringVal::StringVal(const string& s) : StringVal(s.length(), s.data()) { }
+StringVal::StringVal(std::string_view s) : StringVal(s.length(), s.data()) { }
 
 StringVal::~StringVal()
 	{
@@ -953,17 +959,17 @@ ValPtr StringVal::SizeVal() const
 	return val_mgr->Count(string_val->Len());
 	}
 
-int StringVal::Len()
+int StringVal::Len() const
 	{
 	return AsString()->Len();
 	}
 
-const u_char* StringVal::Bytes()
+const u_char* StringVal::Bytes() const
 	{
 	return AsString()->Bytes();
 	}
 
-const char* StringVal::CheckString()
+const char* StringVal::CheckString() const
 	{
 	return AsString()->CheckString();
 	}
@@ -1316,6 +1322,16 @@ ValPtr ListVal::DoClone(CloneState* state)
 	return lv;
 	}
 
+unsigned int ListVal::ComputeFootprint(std::unordered_set<const Val*>* analyzed_vals) const
+	{
+	unsigned int fp = vals.size();
+
+	for ( const auto& val : vals )
+		fp += val->Footprint(analyzed_vals);
+
+	return fp;
+	}
+
 unsigned int ListVal::MemoryAllocation() const
 	{
 #pragma GCC diagnostic push
@@ -1343,15 +1359,27 @@ TableValTimer::TableValTimer(TableVal* val, double t) : detail::Timer(t, detail:
 
 TableValTimer::~TableValTimer()
 	{
-	table->ClearTimer(this);
+	if ( table )
+		table->ClearTimer(this);
 	}
 
 void TableValTimer::Dispatch(double t, bool is_expire)
 	{
 	if ( ! is_expire )
 		{
+		// Take this reference in case the expiration does something silly like resetting the
+		// table object itself. Doing so would cause a crash since the method would try to
+		// delete the table while it was being actively used.
+		TableValPtr temp = {NewRef{}, table};
+
 		table->ClearTimer(this);
 		table->DoExpire(t);
+
+		// If the table did get deleted earlier, then the only existing reference will be the
+		// one taken above. In that case, set table to nullptr here so ~TableValTimer doesn't
+		// also try to do something with it.
+		if ( table->RefCnt() == 1 )
+			table = nullptr;
 		}
 	}
 
@@ -1361,7 +1389,10 @@ static void table_entry_val_delete_func(void* val)
 	delete tv;
 	}
 
-static void find_nested_record_types(const TypePtr& t, std::set<RecordType*>* found)
+// Third argument tracks records currently being analyzed, to avoid infinite
+// loops in the face of recursive records.
+static void find_nested_record_types(const TypePtr& t, std::set<RecordType*>* found,
+                                     std::set<const RecordType*>* analyzed_records)
 	{
 	if ( ! t )
 		return;
@@ -1371,31 +1402,38 @@ static void find_nested_record_types(const TypePtr& t, std::set<RecordType*>* fo
 		case TYPE_RECORD:
 			{
 			auto rt = t->AsRecordType();
+
+			if ( analyzed_records->count(rt) > 0 )
+				return;
+
+			analyzed_records->insert(rt);
 			found->emplace(rt);
 
 			for ( auto i = 0; i < rt->NumFields(); ++i )
-				find_nested_record_types(rt->FieldDecl(i)->type, found);
+				find_nested_record_types(rt->FieldDecl(i)->type, found, analyzed_records);
+
+			analyzed_records->erase(rt);
 			}
 			return;
 		case TYPE_TABLE:
-			find_nested_record_types(t->AsTableType()->GetIndices(), found);
-			find_nested_record_types(t->AsTableType()->Yield(), found);
+			find_nested_record_types(t->AsTableType()->GetIndices(), found, analyzed_records);
+			find_nested_record_types(t->AsTableType()->Yield(), found, analyzed_records);
 			return;
 		case TYPE_LIST:
 			{
 			for ( const auto& type : t->AsTypeList()->GetTypes() )
-				find_nested_record_types(type, found);
+				find_nested_record_types(type, found, analyzed_records);
 			}
 			return;
 		case TYPE_FUNC:
-			find_nested_record_types(t->AsFuncType()->Params(), found);
-			find_nested_record_types(t->AsFuncType()->Yield(), found);
+			find_nested_record_types(t->AsFuncType()->Params(), found, analyzed_records);
+			find_nested_record_types(t->AsFuncType()->Yield(), found, analyzed_records);
 			return;
 		case TYPE_VECTOR:
-			find_nested_record_types(t->AsVectorType()->Yield(), found);
+			find_nested_record_types(t->AsVectorType()->Yield(), found, analyzed_records);
 			return;
 		case TYPE_TYPE:
-			find_nested_record_types(t->AsTypeType()->GetType(), found);
+			find_nested_record_types(t->AsTypeType()->GetType(), found, analyzed_records);
 			return;
 		default:
 			return;
@@ -1413,9 +1451,10 @@ TableVal::TableVal(TableTypePtr t, detail::AttributesPtr a) : Val(t)
 	for ( const auto& t : table_type->GetIndexTypes() )
 		{
 		std::set<RecordType*> found;
+		std::set<const RecordType*> analyzed_records;
 		// TODO: this likely doesn't have to be repeated for each new TableVal,
 		//       can remember the resulting dependencies per TableType
-		find_nested_record_types(t, &found);
+		find_nested_record_types(t, &found, &analyzed_records);
 
 		for ( auto rt : found )
 			parse_time_table_record_dependencies[rt].emplace_back(NewRef{}, this);
@@ -1770,57 +1809,6 @@ bool TableVal::IsSubsetOf(const TableVal& tv) const
 	return true;
 	}
 
-bool TableVal::ExpandAndInit(ValPtr index, ValPtr new_val)
-	{
-	const auto& index_type = index->GetType();
-
-	if ( index_type->IsSet() )
-		{
-		index = index->AsTableVal()->ToListVal();
-		return ExpandAndInit(std::move(index), std::move(new_val));
-		}
-
-	if ( index_type->Tag() != TYPE_LIST )
-		// Nothing to expand.
-		return CheckAndAssign(std::move(index), std::move(new_val));
-
-	ListVal* iv = index->AsListVal();
-	if ( iv->BaseTag() != TYPE_ANY )
-		{
-		if ( table_type->GetIndices()->GetTypes().size() != 1 )
-			reporter->InternalError("bad singleton list index");
-
-		for ( int i = 0; i < iv->Length(); ++i )
-			if ( ! ExpandAndInit(iv->Idx(i), new_val) )
-				return false;
-
-		return true;
-		}
-
-	else
-		{ // Compound table.
-		int i;
-
-		for ( i = 0; i < iv->Length(); ++i )
-			{
-			const auto& v = iv->Idx(i);
-			// ### if CompositeHash::ComputeHash did flattening
-			// of 1-element lists (like ComputeSingletonHash does),
-			// then we could optimize here.
-			const auto& t = v->GetType();
-
-			if ( t->IsSet() || t->Tag() == TYPE_LIST )
-				break;
-			}
-
-		if ( i >= iv->Length() )
-			// Nothing to expand.
-			return CheckAndAssign(std::move(index), std::move(new_val));
-		else
-			return ExpandCompoundAndInit(iv, i, std::move(new_val));
-		}
-	}
-
 ValPtr TableVal::Default(const ValPtr& index)
 	{
 	const auto& def_attr = GetAttr(detail::ATTR_DEFAULT);
@@ -2145,15 +2133,7 @@ void TableVal::SendToStore(const Val* index, const TableEntryVal* new_entry_val,
 			case ELEMENT_NEW:
 			case ELEMENT_CHANGED:
 				{
-#ifndef __clang__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
-#endif
-				broker::optional<broker::timespan> expiry;
-#ifndef __clang__
-#pragma GCC diagnostic pop
-#endif
-
+				std::optional<broker::timespan> expiry;
 				auto expire_time = GetExpireTime();
 				if ( expire_time == 0 )
 					// Entry is set to immediately expire. Let's not forward it.
@@ -2456,49 +2436,6 @@ void TableVal::Describe(ODesc* d) const
 		}
 	}
 
-bool TableVal::ExpandCompoundAndInit(ListVal* lv, int k, ValPtr new_val)
-	{
-	Val* ind_k_v = lv->Idx(k).get();
-	auto ind_k = ind_k_v->GetType()->IsSet() ? ind_k_v->AsTableVal()->ToListVal()
-	                                         : ListValPtr{NewRef{}, ind_k_v->AsListVal()};
-
-	for ( int i = 0; i < ind_k->Length(); ++i )
-		{
-		const auto& ind_k_i = ind_k->Idx(i);
-		auto expd = make_intrusive<ListVal>(TYPE_ANY);
-
-		for ( auto j = 0; j < lv->Length(); ++j )
-			{
-			const auto& v = lv->Idx(j);
-
-			if ( j == k )
-				expd->Append(ind_k_i);
-			else
-				expd->Append(v);
-			}
-
-		if ( ! ExpandAndInit(std::move(expd), new_val) )
-			return false;
-		}
-
-	return true;
-	}
-
-bool TableVal::CheckAndAssign(ValPtr index, ValPtr new_val)
-	{
-	Val* v = nullptr;
-	if ( subnets )
-		// We need an exact match here.
-		v = (Val*)subnets->Lookup(index.get(), true);
-	else
-		v = Find(index).get();
-
-	if ( v )
-		index->Warn("multiple initializations for index");
-
-	return Assign(std::move(index), std::move(new_val));
-	}
-
 void TableVal::InitDefaultFunc(detail::Frame* f)
 	{
 	// Value aready initialized.
@@ -2769,6 +2706,24 @@ ValPtr TableVal::DoClone(CloneState* state)
 	return tv;
 	}
 
+unsigned int TableVal::ComputeFootprint(std::unordered_set<const Val*>* analyzed_vals) const
+	{
+	unsigned int fp = table_val->Length();
+
+	for ( const auto& iter : *table_val )
+		{
+		auto k = iter.GetHashKey();
+		auto vl = table_hash->RecoverVals(*k);
+		auto v = iter.GetValue<TableEntryVal*>()->GetVal();
+
+		fp += vl->Footprint(analyzed_vals);
+		if ( v )
+			fp += v->Footprint(analyzed_vals);
+		}
+
+	return fp;
+	}
+
 unsigned int TableVal::MemoryAllocation() const
 	{
 	unsigned int size = 0;
@@ -2977,13 +2932,12 @@ ValPtr RecordVal::GetFieldOrDefault(const char* field) const
 	return GetFieldOrDefault(idx);
 	}
 
-RecordValPtr RecordVal::CoerceTo(RecordTypePtr t, RecordValPtr aggr, bool allow_orphaning) const
+RecordValPtr RecordVal::DoCoerceTo(RecordTypePtr t, bool allow_orphaning) const
 	{
 	if ( ! record_promotion_compatible(t.get(), GetType()->AsRecordType()) )
 		return nullptr;
 
-	if ( ! aggr )
-		aggr = make_intrusive<RecordVal>(std::move(t));
+	auto aggr = make_intrusive<RecordVal>(std::move(t));
 
 	RecordType* ar_t = aggr->GetType()->AsRecordType();
 	const RecordType* rv_t = GetType()->AsRecordType();
@@ -3041,7 +2995,7 @@ RecordValPtr RecordVal::CoerceTo(RecordTypePtr t, bool allow_orphaning)
 	if ( same_type(GetType(), t) )
 		return {NewRef{}, this};
 
-	return CoerceTo(std::move(t), nullptr, allow_orphaning);
+	return DoCoerceTo(std::move(t), allow_orphaning);
 	}
 
 TableValPtr RecordVal::GetRecordFieldsVal() const
@@ -3135,6 +3089,24 @@ ValPtr RecordVal::DoClone(CloneState* state)
 	return rv;
 	}
 
+unsigned int RecordVal::ComputeFootprint(std::unordered_set<const Val*>* analyzed_vals) const
+	{
+	int n = NumFields();
+	unsigned int fp = n;
+
+	for ( auto i = 0; i < n; ++i )
+		{
+		if ( ! HasField(i) )
+			continue;
+
+		auto f_i = GetField(i);
+		if ( f_i )
+			fp += f_i->Footprint(analyzed_vals);
+		}
+
+	return fp;
+	}
+
 unsigned int RecordVal::MemoryAllocation() const
 	{
 	unsigned int size = 0;
@@ -3334,19 +3306,10 @@ bool VectorVal::Insert(unsigned int index, ValPtr element)
 	auto n = vector_val->size();
 
 	if ( index < n )
-		{ // May need to delete previous element
+		{ // Find location within existing vector elements.
 		it = std::next(vector_val->begin(), index);
 		if ( yield_types )
-			{
-			if ( *it )
-				ZVal::DeleteIfManaged(**it, element->GetType());
 			types_it = std::next(yield_types->begin(), index);
-			}
-		else if ( managed_yield )
-			{
-			if ( *it )
-				ZVal::DeleteManagedType(**it);
-			}
 		}
 	else
 		{
@@ -3667,6 +3630,21 @@ bool VectorVal::Concretize(const TypePtr& t)
 	return true;
 	}
 
+unsigned int VectorVal::ComputeFootprint(std::unordered_set<const Val*>* analyzed_vals) const
+	{
+	auto n = vector_val->size();
+	unsigned int fp = n;
+
+	for ( auto i = 0U; i < n; ++i )
+		{
+		auto v = At(i);
+		if ( v )
+			fp += v->Footprint(analyzed_vals);
+		}
+
+	return fp;
+	}
+
 unsigned int VectorVal::Resize(unsigned int new_num_elements)
 	{
 	unsigned int oldsize = vector_val->size();
@@ -3741,8 +3719,19 @@ void VectorVal::ValDescribe(ODesc* d) const
 	d->Add("]");
 	}
 
-ValPtr check_and_promote(ValPtr v, const Type* t, bool is_init,
+ValPtr check_and_promote(ValPtr v, const TypePtr& t, bool is_init,
                          const detail::Location* expr_location)
+	{
+		// Once 5.0 comes out, this function can merge with the deprecated one below it, and this
+		// pragma block can go away.
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+	return check_and_promote(v, t.get(), is_init, expr_location);
+#pragma GCC diagnostic pop
+	}
+
+[[deprecated("Remove in v5.1. Use version that takes TypePtr instead.")]] ValPtr
+check_and_promote(ValPtr v, const Type* t, bool is_init, const detail::Location* expr_location)
 	{
 	if ( ! v )
 		return nullptr;
@@ -4037,6 +4026,31 @@ ValPtr Val::MakeCount(bro_uint_t u)
 	return make_intrusive<CountVal>(u);
 	}
 
+unsigned int Val::Footprint(std::unordered_set<const Val*>* analyzed_vals) const
+	{
+	auto is_aggr = IsAggr(type);
+
+	// We only need to check containers for possible recursion, as there's
+	// no way to construct a cycle using only non-aggregates.
+	if ( is_aggr )
+		{
+		if ( analyzed_vals->count(this) > 0 )
+			// Footprint is 1 for generating a cycle.
+			return 1;
+
+		analyzed_vals->insert(this);
+		}
+
+	auto fp = ComputeFootprint(analyzed_vals);
+
+	if ( is_aggr )
+		// Allow the aggregate to be revisited providing it's not
+		// in the context of a cycle.
+		analyzed_vals->erase(this);
+
+	return fp;
+	}
+
 ValManager::ValManager()
 	{
 	empty_string = make_intrusive<StringVal>("");
