@@ -410,12 +410,10 @@ const ZAMStmt ZAMCompiler::CompileSwitch(const SwitchStmt* sw)
 	// Need to track a new set of contexts for "break" statements.
 	PushBreaks();
 
-	auto& cases = *sw->Cases();
-
-	if ( cases.length() > 0 && cases[0]->TypeCases() )
-		return TypeSwitch(sw, n, c);
-	else
+	if ( sw->TypeMap()->empty() )
 		return ValueSwitch(sw, n, c);
+	else
+		return TypeSwitch(sw, n, c);
 	}
 
 const ZAMStmt ZAMCompiler::ValueSwitch(const SwitchStmt* sw, const NameExpr* v, const ConstExpr* c)
@@ -713,9 +711,8 @@ const ZAMStmt ZAMCompiler::CompileDel(const DelStmt* ds)
 	if ( index_list->Tag() != EXPR_LIST )
 		reporter->InternalError("non-list in \"delete\"");
 
-	auto internal_ind = BuildVals(index_list->AsListExprPtr());
-
-	return DelTableVO(aggr, internal_ind);
+	auto internal_ind = std::unique_ptr<OpaqueVals>(BuildVals(index_list->AsListExprPtr()));
+	return DelTableVO(aggr, internal_ind.get());
 	}
 
 const ZAMStmt ZAMCompiler::CompileWhile(const WhileStmt* ws)
@@ -1115,7 +1112,7 @@ const ZAMStmt ZAMCompiler::CompileWhen(const WhenStmt* ws)
 		z.v1 = is_return;
 		}
 
-	z.e = cond;
+	z.e = cond.get();
 
 	auto when_eval = AddInst(z);
 
